package fr.faraphel.m1_pe_kafka

import fr.faraphel.m1_pe_kafka.kafka.AdminUtils
import fr.faraphel.m1_pe_kafka.kafka.PrintConsumer
import fr.faraphel.m1_pe_kafka.kafka.Converter
import fr.faraphel.m1_pe_kafka.kafka.TemperatureProducer
import fr.faraphel.m1_pe_kafka.utils.celsius
import io.quarkus.runtime.Quarkus
import io.quarkus.runtime.QuarkusApplication
import io.quarkus.runtime.annotations.QuarkusMain


val TOPIC_TEMPERATURE_CELSIUS: String = "temperature-celsius"
val TOPIC_TEMPERATURE_FAHRENHEIT: String = "temperature-fahrenheit"


/**
 * The entrypoint of the program
 */
@QuarkusMain
class Main : QuarkusApplication {
    override fun run(vararg args: String?): Int {
        println("starting...")

        // create an admin object
        val admin = AdminUtils()
        // create our topics
        admin.createTopics("temperature-celsius", "temperature-fahrenheit")

        // create a producer that will generate temperature values based on the current temperature at Amiens (France).
        val producer = TemperatureProducer(
            latitude=49.9,
            longitude=2.3,
            topic=TOPIC_TEMPERATURE_CELSIUS
        )
        // create a converter that will convert values coming from the Celsius topic to the Fahrenheit topic
        val converter = Converter(
            inputTopic=TOPIC_TEMPERATURE_CELSIUS,
            outputTopic=TOPIC_TEMPERATURE_FAHRENHEIT
        ) { temperature -> temperature.celsius.asFahrenheit }
        // create a consumer that will print the received values in the Fahrenheit topic
        val consumer = PrintConsumer(
            topic=TOPIC_TEMPERATURE_FAHRENHEIT
        )

        // wrap the producer and the consumer in threads
        // TODO(Faraphel): should be native to the producer and consumer class ?
        val threadProducer = Thread { producer.produceForever() }
        val threadConsumer = Thread { consumer.consumeForever() }

        // run all the clients
        threadConsumer.start()
        converter.start()
        threadProducer.start()

        // wait for them to finish before closing
        threadConsumer.join()
        threadProducer.join()
        converter.stop()

        return 0
    }
}

/**
 * The main function
 * Call the entrypoint directly. Used to run the program directly with Kotlin
 */
fun main(args: Array<String>) {
    Quarkus.run(Main::class.java, *args)
}
